@@ -1,24 +1,40 @@
-from typing import List, Union, TypeVar, Type, Tuple
+from dataclasses import dataclass, field
+from typing import List, TypeVar, Tuple, Generic
 
 from pydantic import BaseModel as PdModel
 from tortoise.exceptions import IntegrityError, FieldError
-from tortoise.queryset import QuerySetSingle, QuerySet
-from web_foundation.app.infrastructure.database.models import AbstractDbModel
+from tortoise.queryset import QuerySet, MODEL
+
+from web_foundation.app.infrastructure.database.models import GenericDbType
 from web_foundation.app.infrastructure.database.utils import integrity_error_format
 from web_foundation.workers.web.erorrs.application import InconsistencyError
 from web_foundation.workers.web.sanic_ext.structs import ProtectIdentity
 
 EntityId = TypeVar("EntityId", bound=int)
-TypeDbModel = TypeVar("TypeDbModel", bound=AbstractDbModel)
+
+CheckNoneType = TypeVar("CheckNoneType")
+
+
+@dataclass
+class AccessProtectIdentity(Generic[GenericDbType], ProtectIdentity):
+    entity_id: int | None = field(default=None)
 
 
 class DatabaseMiddleware:
     pass
 
 
+def avoid_none(instance: CheckNoneType | None) -> CheckNoneType:
+    if not instance:
+        raise Exception("Need instance not found")
+    else:
+        return instance
+
+
 class RelateDMW(DatabaseMiddleware):
     @staticmethod
-    async def fetch_related(model: TypeDbModel, query: Union[QuerySet, QuerySetSingle], fetch_fields: List[str]):
+    async def fetch_related(model: GenericDbType, query: QuerySet[MODEL],
+                            fetch_fields: List[str] | None = None) -> QuerySet[MODEL]:
         if fetch_fields:
             for_select = {field for fk_field in model._meta.fk_fields for field in fetch_fields if
                           fk_field == field}
@@ -31,20 +47,18 @@ class RelateDMW(DatabaseMiddleware):
 
 
 class AccessDMW(DatabaseMiddleware):
-    async def get_entity(self, model: Type[TypeDbModel], user, entity_id: EntityId) -> TypeDbModel: # TODO TYPED USER
-        if "user" in model._meta.fields:
-            entity = await model.get_or_none(id=entity_id, user_id=user.id)
-        else:
-            entity = await model.get_or_none(id=entity_id)
+    @staticmethod
+    async def get_entity(model: GenericDbType,
+                         entity_id: EntityId) -> GenericDbType:  # TODO TYPED USER
+        entity = await model.get_or_none(id=entity_id)
         if not entity:
-            raise InconsistencyError(message=f"{model.__name__} not found")
+            raise InconsistencyError(message=f"{model.__name__} not found")  # type: ignore
         return entity
 
     @staticmethod
-    async def create(model: TypeDbModel, protection: ProtectIdentity, dto: PdModel, **kwargs) -> TypeDbModel:
-        entity_kwargs = {field: value for field, value in dto.dict().items() if not value is None}
-        if "user" in model._meta.fields:
-            entity_kwargs["user"] = protection.user
+    async def create(model: GenericDbType, protection: AccessProtectIdentity, dto: PdModel | None,
+                     **kwargs) -> GenericDbType:
+        entity_kwargs = {field: value for field, value in avoid_none(dto).dict().items() if value is not None}
         try:
             entity = await model.create(**entity_kwargs)
         except IntegrityError as exception:
@@ -54,9 +68,8 @@ class AccessDMW(DatabaseMiddleware):
         return entity
 
     @staticmethod
-    async def read(model: TypeDbModel, protection: ProtectIdentity, fetch_fields: List[str] = None, **kwargs) -> \
-            Union[
-                TypeDbModel, None]:
+    async def read(model: GenericDbType, protection: AccessProtectIdentity, fetch_fields: List[str] | None = None,
+                   **kwargs) -> GenericDbType | None:
         query = model.filter(id=protection.entity_id)
         if kwargs:
             query = query.filter(**kwargs)
@@ -65,9 +78,9 @@ class AccessDMW(DatabaseMiddleware):
         return entity
 
     @staticmethod
-    async def read_all(model: TypeDbModel, protection: ProtectIdentity, limit: int = None, offset: int = None,
+    async def read_all(model: GenericDbType, protection: AccessProtectIdentity, limit: int = None, offset: int = None,
                        fetch_fields: List[str] = None,
-                       **kwargs) -> Union[Tuple[List[TypeDbModel], int], TypeDbModel]:
+                       **kwargs) -> Tuple[List[GenericDbType], int]:
         query = model.filter()
         if "banned" in model._meta.fields:
             query = query.filter(banned=False)
@@ -90,9 +103,10 @@ class AccessDMW(DatabaseMiddleware):
         return entities, total
 
     @staticmethod
-    async def update(model: TypeDbModel, protection: ProtectIdentity, dto: PdModel, **kwargs) -> TypeDbModel:
-        entity = await AccessDMW.get_entity(model, protection.user, protection.entity_id)
-        entity_kwargs = {field: value for field, value in dto.dict().items() if value is not None}
+    async def update(model: GenericDbType, protection: AccessProtectIdentity, dto: PdModel | None,
+                     **kwargs) -> GenericDbType:
+        entity = await AccessDMW.get_entity(model, avoid_none(protection.entity_id))
+        entity_kwargs = {field: value for field, value in avoid_none(dto).dict().items() if value is not None}
         for field, value in entity_kwargs.items():
             setattr(entity, field, value)
         try:
@@ -104,7 +118,7 @@ class AccessDMW(DatabaseMiddleware):
         return entity
 
     @staticmethod
-    async def delete(model: TypeDbModel, protection: ProtectIdentity, **kwargs) -> TypeDbModel:
-        entity = await AccessDMW.get_entity(model, protection.user, protection.entity_id)
+    async def delete(model: GenericDbType, protection: AccessProtectIdentity, **kwargs) -> GenericDbType:
+        entity = await AccessDMW.get_entity(model, avoid_none(protection.entity_id))
         await entity.delete()
         return entity
